/****************************************************************************
** Copyright (C) 2020 MikroElektronika d.o.o.
** Contact: https://www.mikroe.com/contact
**
** Permission is hereby granted, free of charge, to any person obtaining a copy
** of this software and associated documentation files (the "Software"), to deal
** in the Software without restriction, including without limitation the rights
** to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
** copies of the Software, and to permit persons to whom the Software is
** furnished to do so, subject to the following conditions:
** The above copyright notice and this permission notice shall be
** included in all copies or substantial portions of the Software.
**
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
** EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
** OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
** DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
**  USE OR OTHER DEALINGS IN THE SOFTWARE.
****************************************************************************/

/*!
 * @file mosaic.c
 * @brief Mosaic Click Driver.
 */

#include "mosaic.h"
#include "generic_pointer.h"

void mosaic_cfg_setup ( mosaic_cfg_t *cfg ) 
{
    // Communication gpio pins
    cfg->rx_pin = HAL_PIN_NC;
    cfg->tx_pin = HAL_PIN_NC;

    // Additional gpio pins
    cfg->rst = HAL_PIN_NC;
    cfg->cts = HAL_PIN_NC;
    cfg->rdy = HAL_PIN_NC;
    cfg->rts = HAL_PIN_NC;

    cfg->baud_rate     = 115200;
    cfg->data_bit      = UART_DATA_BITS_DEFAULT;
    cfg->parity_bit    = UART_PARITY_DEFAULT;
    cfg->stop_bit      = UART_STOP_BITS_DEFAULT;
    cfg->uart_blocking = false;
}

err_t mosaic_init ( mosaic_t *ctx, mosaic_cfg_t *cfg ) 
{
    uart_config_t uart_cfg;

    // Default config
    uart_configure_default( &uart_cfg );

    // Ring buffer mapping
    ctx->uart.tx_ring_buffer = ctx->uart_tx_buffer;
    ctx->uart.rx_ring_buffer = ctx->uart_rx_buffer;

    // UART module config
    uart_cfg.rx_pin = cfg->rx_pin;  // UART RX pin.
    uart_cfg.tx_pin = cfg->tx_pin;  // UART TX pin.
    uart_cfg.tx_ring_size = sizeof( ctx->uart_tx_buffer );
    uart_cfg.rx_ring_size = sizeof( ctx->uart_rx_buffer );

    if ( UART_ERROR == uart_open( &ctx->uart, &uart_cfg ) ) 
    {
        return UART_ERROR;
    }
    uart_set_baud( &ctx->uart, cfg->baud_rate );
    uart_set_parity( &ctx->uart, cfg->parity_bit );
    uart_set_stop_bits( &ctx->uart, cfg->stop_bit );
    uart_set_data_bits( &ctx->uart, cfg->data_bit );

    uart_set_blocking( &ctx->uart, cfg->uart_blocking );

    // Output pins
    digital_out_init( &ctx->rst, cfg->rst );
    digital_out_init( &ctx->cts, cfg->cts );
    digital_out_high( &ctx->rst );
    digital_out_low( &ctx->cts );

    // Input pins
    digital_in_init( &ctx->rdy, cfg->rdy );
    digital_in_init( &ctx->rts, cfg->rts );

    Delay_100ms ( );

    return UART_SUCCESS;
}

err_t mosaic_generic_write ( mosaic_t *ctx, uint8_t *data_in, uint16_t len ) 
{
    return uart_write( &ctx->uart, data_in, len );
}

err_t mosaic_generic_read ( mosaic_t *ctx, uint8_t *data_out, uint16_t len ) 
{
    return uart_read( &ctx->uart, data_out, len );
}

void mosaic_set_rst_pin ( mosaic_t *ctx, uint8_t state )
{
    digital_out_write ( &ctx->rst, state );
}

void mosaic_set_cts_pin ( mosaic_t *ctx, uint8_t state )
{
    digital_out_write ( &ctx->cts, state );
}

uint8_t mosaic_get_rdy_pin ( mosaic_t *ctx )
{
    return digital_in_read ( &ctx->rdy );
}

uint8_t mosaic_get_rts_pin ( mosaic_t *ctx )
{
    return digital_in_read ( &ctx->rts );
}

void mosaic_send_cmd ( mosaic_t *ctx, uint8_t *cmd )
{
    while ( *cmd != 0 )
    {
        uart_write( &ctx->uart, cmd, 1 );
        cmd++;
    }
    uart_write( &ctx->uart, MOSAIC_END_CMD, 2 );
    Delay_100ms(  );
}

void mosaic_send_cmd_with_par ( mosaic_t *ctx, uint8_t *cmd_buf, uint8_t *param_buf )
{
    uint8_t final_cmd[ MOSAIC_TX_DRV_BUFFER_SIZE ] = { 0 };
    
    strcpy( final_cmd, cmd_buf );
    strcat( final_cmd, MOSAIC_PARAM_DELIMITER );
    strcat( final_cmd, param_buf );
    
    mosaic_send_cmd( ctx, final_cmd );
}

void mosaic_enable_nmea_output ( mosaic_t *ctx, uint8_t *msg_param, uint8_t *interval )
{
    uint8_t param_buf[ MOSAIC_TX_DRV_BUFFER_SIZE ] = { 0 };
    strcpy( param_buf, MOSAIC_SNO_STREAM_1 );
    strcat( param_buf, MOSAIC_PARAM_DELIMITER );
    strcat( param_buf, MOSAIC_SNO_CD_COM1 );
    strcat( param_buf, MOSAIC_PARAM_DELIMITER );
    strcat( param_buf, msg_param );
    strcat( param_buf, MOSAIC_PARAM_DELIMITER );
    strcat( param_buf, interval );
    mosaic_send_cmd_with_par( ctx, MOSAIC_CMD_SET_NMEA_OUTPUT, param_buf );
}

void mosaic_disable_nmea_output ( mosaic_t *ctx )
{
    uint8_t param_buf[ MOSAIC_TX_DRV_BUFFER_SIZE ] = { 0 };
    strcpy( param_buf, MOSAIC_SNO_STREAM_1 );
    strcat( param_buf, MOSAIC_PARAM_DELIMITER );
    strcat( param_buf, MOSAIC_SNO_CD_NONE );
    strcat( param_buf, MOSAIC_PARAM_DELIMITER );
    strcat( param_buf, MOSAIC_SNO_MESSAGES_NONE );
    strcat( param_buf, MOSAIC_PARAM_DELIMITER );
    strcat( param_buf, MOSAIC_SNO_INTERVAL_OFF );
    mosaic_send_cmd_with_par( ctx, MOSAIC_CMD_SET_NMEA_OUTPUT, param_buf );
}

err_t mosaic_parse_gga ( uint8_t *rsp_buf, uint8_t gga_element, uint8_t *element_data )
{    
    if ( gga_element > MOSAIC_GGA_ELEMENT_SIZE )
    {
        return MOSAIC_ERROR;
    }
    uint8_t * __generic_ptr start_ptr = strstr( rsp_buf, MOSAIC_NMEA_GGA );
    if ( start_ptr )
    {
        if ( strstr( start_ptr, MOSAIC_NMEA_END ) )
        {
            for ( uint8_t element_cnt = 0; element_cnt < gga_element; element_cnt++ )
            {
                start_ptr = strstr( start_ptr, MOSAIC_NMEA_DELIMITER );
                if ( start_ptr )
                {
                    start_ptr++;
                }
                else
                {
                    return MOSAIC_ERROR;
                }
            }
            uint8_t * __generic_ptr end_ptr = strstr( start_ptr, MOSAIC_NMEA_DELIMITER );
            if ( end_ptr )
            {
                strncpy( element_data, start_ptr, end_ptr - start_ptr );
                return MOSAIC_OK;
            }
        }
    }
    return MOSAIC_ERROR;
}

// ------------------------------------------------------------------------- END
